import { createSlice, PayloadAction } from '@reduxjs/toolkit';
import type { RootState } from './store';
import { AllLayoutAlgorithms, Layouts } from '@graphpolaris/shared/lib/graph-layout';
import { SchemaUtils } from '../../schema/schema-utils';
import { SchemaFromBackend, SchemaGraph, SchemaGraphology } from '../../schema';

/**************************************************************** */

export type SchemaSettings = {
  connectionType: 'connection' | 'bezier' | 'straight' | 'step';
};

type schemaSliceI = {
  graph: SchemaGraph;
  layoutName: AllLayoutAlgorithms;
  settings: SchemaSettings;
};

// Define the initial state using that type
export const initialState: schemaSliceI = {
  graph: new SchemaGraphology().export(),
  // layoutName: 'Cytoscape_fcose',
  layoutName: Layouts.KLAY,
  settings: {
    connectionType: 'connection',
  },
};
export const schemaSlice = createSlice({
  name: 'schema',
  // `createSlice` will infer the state type from the `initialState` argument
  initialState,
  reducers: {
    setSchema: (state, action: PayloadAction<SchemaGraph>) => {
      console.log('setSchema', action);
      state.graph = action.payload;
    },

    setSchemaLayout: (state, action: PayloadAction<string>) => {
      console.log('setSchemaLayout', action);
      state.layoutName = action.payload as AllLayoutAlgorithms;
    },

    readInSchemaFromBackend: (state, action: PayloadAction<SchemaFromBackend>) => {
      state.graph = SchemaUtils.schemaBackend2Graphology(action.payload).export();
      // console.log('Updated schema from backend');
      // The graph schema needs a node for each node AND edge. These need then be connected

      // nodes.forEach((node) => {
      //   schema.addNode(node.name, {
      //     name: node.name,
      //     attributes: node.attributes,
      //     x: 0,
      //     y: 0,
      //   });
      // });

      // // The name of the edge will be name + from + to, since edge names are not unique
      // edges.forEach((edge) => {
      //   const edgeID = edge.name + edge.from + edge.to;

      //   // This node is the actual edge
      //   schema.addNode(edgeID, {
      //     name: edge.name,
      //     attributes: edge.attributes,
      //     from: edge.from,
      //     to: edge.to,
      //     collection: edge.collection,
      //     x: 0,
      //     y: 0,
      //   });

      //   // These lines are simply for keeping the schema together
      //   schema.addDirectedEdgeWithKey(edgeID + 'f', edge.from, edgeID);
      //   schema.addDirectedEdgeWithKey(edgeID + 't', edgeID, edge.to);
      // });

      // state.graphologySerialized = schema.export();
    },

    setSchemaSettings: (state, action: PayloadAction<SchemaSettings>) => {
      state.settings = action.payload;
    },
  },
});
export const { readInSchemaFromBackend, setSchema, setSchemaSettings } = schemaSlice.actions;

export const schemaSettingsState = (state: RootState) => state.schema.settings;

/**
 * Select the schema and convert it to a graphology object
 * */
export const toSchemaGraphology = (graph: SchemaGraph): SchemaGraphology => {
  // This is really weird but for some reason all the attributes appeared as read-only otherwise
  let ret = new SchemaGraphology();
  ret.import(SchemaGraphology.from(graph).export());
  return ret;
};

/**
 * Select the schema
 * */
export const schemaGraph = (state: RootState): SchemaGraph => {
  return state.schema.graph;
};

// /**
//  * selects the SchemaLayout enum
//  * @param {GraphLayout} state
//  * @returns {GraphLayout} enum of type GraphLayout
//  */
export const selectSchemaLayout = (state: RootState) => state.schema.layoutName;

export default schemaSlice.reducer;
